static char sccsid[]="@(#)94    1.10 src/ivauthn/modules/pdxauthn/pdxauthn_adk/xauthn.c, pdweb.authn, pdweb390, 020409a 3/26/02 16:26:07";
/*
 * FILE: xauthn.c
 *
 * PD cross domain authentication (CDAS) demo.  This file implements
 *
 *     xauthn_initialize()
 *     xauthn_shutdown()
 *     xauthn_authenticate()
 *     xauthn_change_password()
 *
 * functions used by the PD WebSEAL to authenticate
 * users based on the specified authentication 
 * mechanism.
 *
 * To configure, modify the iv.conf file, under the 
 * [authentication-mechanisms] stanza, select the desired 
 * authentication mechanism that you want this library to
 * be used, and assign this library to it.
 *
 * For example, if you wish this library to process all the
 * HTTP SSL username/password LDAP authentication, specifies 
 * the following:
 *
 *    passwd-ldap = libxauthn.so
 *
 * or
 *
 *    passwd-ldap = libxauthn.so&<args>
 *
 * if you have any particular arguments that you want to pass
 * to this library for initialization and shutdown.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ogauthzn.h>
#if !defined(WIN32) && !defined(_WIN32)
    #include <strings.h>
#endif 

#include "pdxauthn.h"
#include "xattr.h"
#include "xnvlist.h"



int map_error(char *error_val, xauthn_status_t *st);

/*
 * FUNCTION NAME
 *      xauthn_initialize
 *
 * DESCRIPTION
 *      init the authentication service
 *
 * ARGUMENTS
 *      [in] argc The count of arguments to the service.
 *      [in] argv The array of argument strings.
 *
 * RETURN VALUE
 *      XAUTHN_S_COMPLETE on success, error code on failure
 */
xauthn_status_t
xauthn_initialize(
                 int        argc,     /* in */
                 const char **argv    /* in */
                 )
{
    return XAUTHN_S_COMPLETE;
}


/*
 * FUNCTION NAME
 *      xauthn_shutdown
 *
 * DESCRIPTION
 *      Shutdown the authentication service.
 *      The initialization parameters are passed in
 *      again.
 *
 * ARGUMENTS
 *      [in] argc     The count of arguments to the service.
 *      [in] argv     The array of argument strings.
 *
 * RETURN VALUE
 *      XAUTHN_S_COMPLETE
 */
xauthn_status_t
xauthn_shutdown(
               int        argc,     /* in */
               const char **argv    /* in */
               )
{
    return XAUTHN_S_COMPLETE;
}


/*
 * FUNCTION NAME
 *      xauthn_authenticate
 *
 * DESCRIPTION
 *      Examine the received user authentication information, and generate a 
 *      client identity. The received information will vary depending on the
 *      specified authentication mechanism. 
 *
 * ARGUMENTS - IN
 *      authInfo      List of names and set of values containing the user
 *                    authentication data. The pdxauthn.h contains all the
 *                    possible names that could be in this list. The actual
 *                    list of names received will depend on the specified    
 *                    authentication mechanism. 
 *
 * ARGUMENTS - OUT
 *      identity      Pointer to the resulted client identity.
 *
 *      st            Set to XAUTHN_S_COMPLETE, or to an error status indicating
 *                    the nature of the failure.
 */
xauthn_status_t 
xauthn_authenticate(
                   xnvlist_t         *authnInfo,
                   xauthn_identity_t *ident
                   )
{
    int               i;
    char              *username;
    char              *error_val;
    char              **name;
    int               prin_type;
    azn_creds_h_t     creds;
    xnvlist_item_t    *item;
    xauthn_status_t   st;
    azn_string_t      prin_name = NULL;

    if ( authnInfo != NULL ) {
        printf("===============================\n");
        printf("User Authentication Information:\n");
        printf("===============================\n");

        for (i=0; i<authnInfo->length; i++) {
            item = &authnInfo->items[i];
            if ( item->name != NULL) {
                /* If this item is a credential, convert it to an */
                /* azn_creds_h_t and print the principal name from the */
                /* cred */
                if ( strcmp(item->name, XAUTHN_EXISTING_CRED) == 0 ||
                     strcmp(item->name, XAUTHN_ADMIN_CRED) == 0 ) {
                    creds = xauthn_util_entry_to_creds(item->value, item->vlen);
                    printf("%s:\t%u\n", item->name, creds);
                    if ( creds != AZN_C_INVALID_HANDLE ) {
                        azn_creds_get_attr_value_string(creds, 
                             AZN_C_INITIATOR_INDEX, azn_cred_principal_name, 
                             &prin_name);
                        if ( prin_name != NULL ) {
                            printf("Principal name from cred: %s\n", prin_name);
                            azn_release_string(&prin_name);
                            prin_name = NULL;
                        }
                    }
                } else if ( strcmp(item->name, XAUTHN_CERT) == 0 ) {
                    /* This item contains binary data, we can't just print it */
                    printf("%s:\t<binary data> (%d)\n", item->name,item->vlen);
                } else {
                    printf("%s:\t%s (%d)\n", item->name, item->value, 
                           item->vlen);
                }
            }
        }
    }
    else {
        printf("There is NO user authenication information available.\n\n");
    }

    printf("Error status test\n");
    printf("=================\n");
    printf("Enter 0 to skip this test.\n");
    printf("Otherwise enter XAUTHN error code to be returned.\n");
    error_val = (char *)malloc(1024);
    scanf("%s", error_val);

    if ( map_error(error_val, &st) ) {
		free(error_val);
        return st;
    }
	free(error_val);

	do {
       printf("Registry ID numbers:\n");
       printf("  LDAP: %d\n", XAUTHN_PRIN_TYPE_DN);
       printf("  URAF: %d\n", XAUTHN_PRIN_TYPE_URAF); 
       printf("Please enter the registry ID number: ");
       scanf("%d", &prin_type);
       if ( prin_type == XAUTHN_PRIN_TYPE_DN || 
             prin_type == XAUTHN_PRIN_TYPE_URAF) {
           /* Valid principal type */
           break;
       }
       /* Invalid principal type */
       printf("Invalid registry ID, please reenter.\n");
    } while (1);

    if ( prin_type == XAUTHN_PRIN_TYPE_DN ) {
        name = &ident->prin.data.dn;
    } else {
        name = &ident->prin.data.uraf_name;
    }

    /*
     * We only want to get a user name if one has not already been added.
     */
    if (*name == NULL) {

        username = (char *)malloc(1024);

        printf("Enter the user identity: ");
        scanf("%s", username);

        if ( strlen(username) == 0 ) {
            printf("Invalid user identity inputted. Authentication failure.\n");

            st = XAUTHN_S_INVALID_USERNAME;
			free(username);
            return st;
        }

        ident->prin.prin_type = prin_type;
        *name = (char*)username;
    }
    else {
        char *tag = (char *) malloc(1024);
        char *tag_data = (char *) malloc(1024); 

        /*
         * A name is already specified so we just want to add some test tag
         * value data.
         */
         
        printf("using existing name: %s\n\n", *name);

        sprintf(tag, "%s", XAUTHN_TAG_VALUE_KEY_PREFIX);
        printf("Enter the test tag: ");
        fflush(stdout);
        scanf("%s", tag + strlen(XAUTHN_TAG_VALUE_KEY_PREFIX));

        printf("Enter the test tag data: ");
        fflush(stdout);
        scanf("%s", tag_data);

        xattr_set(&ident->xattrs, tag, tag_data);
    }

    return st;
}


xauthn_status_t 
xauthn_change_password(
                      xnvlist_t         *authnInfo
                      )
{
    int               i;
    char              passwdChanged[2];
    xauthn_status_t   st;

    printf("====================\n");
    printf("User Password Change:\n");
    printf("====================\n");

    for (i=0; i<authnInfo->length; i++) {
        if ( ((authnInfo->items) + i)->name != NULL) {
            printf("%s:\t%s (%d)\n", authnInfo->items[i].name,
                   authnInfo->items[i].value,
                   authnInfo->items[i].vlen);
        }
    }

    printf("Does the password change succeed [y/n]? ");
    scanf("%s", passwdChanged);

    if ( *passwdChanged != 'y' &&
         *passwdChanged != 'Y') {
        printf("User password changed failed.\n");

        st = XAUTHN_S_BAD_AUTHENTICATION_INFO;
        return st;
    }

    return XAUTHN_S_COMPLETE;
}

/*
 * Returns TRUE if the error status is fatal, false otherwise
 */
int
map_error(char              *error_val, 
          xauthn_status_t   *st) 
{
    if ( ! error_val || strlen(error_val) == 0 ) {
        printf("Invalid input! Just returning authentication failure.\n");
        *st = XAUTHN_S_AUTHENTICATION_FAILURE;
        return 1;
    }

    *st = XAUTHN_S_COMPLETE;

    if ( strcmp(error_val, "0") == 0 ) {
        printf("Skipping error test.\n\n");
        return 0;
    }

    if ( strcmp(error_val, "XAUTHN_S_INTERNAL_ERROR") == 0 ) {
        *st = XAUTHN_S_INTERNAL_ERROR;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_UNKNOWN_ERROR") == 0 ) {
        *st = XAUTHN_S_UNKNOWN_ERROR;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_MECHANISM_ERROR") == 0 ) {
        *st = XAUTHN_S_MECHANISM_ERROR;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_OUT_OF_MEMORY") == 0 ) {
        *st = XAUTHN_S_OUT_OF_MEMORY;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_OPERATION_ABORTED") == 0 ) {
        *st = XAUTHN_S_OPERATION_ABORTED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_UNKNOWN_IDENTITY_TYPE") == 0 ) {
        *st = XAUTHN_S_UNKNOWN_IDENTITY_TYPE;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_CANT_LOAD_XATTRS") == 0 ) {
        *st = XAUTHN_S_CANT_LOAD_XATTRS;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_CANT_SELECT_MECHANISM") == 0 ) {
        *st = XAUTHN_S_CANT_SELECT_MECHANISM;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_UNSUPPORTED_AUTHN_METHOD") == 0 ) {
        *st = XAUTHN_S_UNSUPPORTED_AUTHN_METHOD;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_MECHANISM_DOWN") == 0 ) {
        *st = XAUTHN_S_MECHANISM_DOWN;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_INVALID_OPERATION") == 0 ) {
        *st = XAUTHN_S_INVALID_OPERATION;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_AUTHENTICATION_FAILURE") == 0 ) {
        *st = XAUTHN_S_AUTHENTICATION_FAILURE;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_BAD_AUTHENTICATION_INFO") == 0 ) {
        *st = XAUTHN_S_BAD_AUTHENTICATION_INFO;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_INVALID_USERNAME") == 0 ) {
        *st = XAUTHN_S_INVALID_USERNAME;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_RETRY_LIMIT_REACHED") == 0 ) {
        *st = XAUTHN_S_RETRY_LIMIT_REACHED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWORD_EXPIRED") == 0 ) {
        /* Non fatal error - authenticate the user, but WebSEAL */
        /* will require them to change their password */
        *st = XAUTHN_S_PASSWORD_EXPIRED;
        return 0;
    }
    else if ( strcmp(error_val, "XAUTHN_S_ACCOUNT_EXPIRED") == 0 ) {
        *st = XAUTHN_S_ACCOUNT_EXPIRED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_LOGIN_POLICY_VIOLATION") == 0 ) {
        *st = XAUTHN_S_LOGIN_POLICY_VIOLATION;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_NEW_PIN") == 0 ) {
        *st = XAUTHN_S_NEW_PIN;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_ACCOUNT_DISABLED") == 0 ) {
        *st = XAUTHN_S_ACCOUNT_DISABLED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_NEXT_TOKEN") == 0 ) {
        *st = XAUTHN_S_NEXT_TOKEN;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_MAPPING_FAILED") == 0 ) {
        *st = XAUTHN_S_MAPPING_FAILED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_CERTIFICATE_AUTH_FAILURE") == 0 ) {
        *st = XAUTHN_S_CERTIFICATE_AUTH_FAILURE;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_HTTP_HEADER_AUTH_FAILURE") == 0 ) {
        *st = XAUTHN_S_HTTP_HEADER_AUTH_FAILURE;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_IP_ADDR_AUTH_FAILED") == 0 ) {
        *st = XAUTHN_S_IP_ADDR_AUTH_FAILED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_SU_INVALID_USERNAME") == 0 ) {
        *st = XAUTHN_S_SU_INVALID_USERNAME;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_SU_NO_PERMS") == 0 ) {
        *st = XAUTHN_S_SU_NO_PERMS;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_POLICY_VIOLATION") == 0 ) {
        *st = XAUTHN_S_PASSWD_POLICY_VIOLATION;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_TOO_SHORT") == 0 ) {
        *st = XAUTHN_S_PASSWD_TOO_SHORT;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_HAS_SPACES") == 0 ) {
        *st = XAUTHN_S_PASSWD_HAS_SPACES;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_TOO_MANY_REPEATED") == 0 ) {
        *st = XAUTHN_S_PASSWD_TOO_MANY_REPEATED;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_TOO_FEW_ALPHAS") == 0 ) {
        *st = XAUTHN_S_PASSWD_TOO_FEW_ALPHAS;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_TOO_FEW_NONALPHAS") == 0 ) {
        *st = XAUTHN_S_PASSWD_TOO_FEW_NONALPHAS;
        return 1;
    }
    else if ( strcmp(error_val, "XAUTHN_S_PASSWD_ACCT_LOCKEDOUT") == 0 ) {
        *st = XAUTHN_S_PASSWD_ACCT_LOCKEDOUT;
        return 1;
    }
    else {
        printf("Invalid input! Just returning authentication failure.\n");
        *st = XAUTHN_S_AUTHENTICATION_FAILURE;
        return 1;
    }
}
    
